"""
Galago Tools - Open Source Lab Automation GRPC Library
"""

from .version import __version__, VERSION_FULL, VERSION_SHORT, VERSION_MAJOR, VERSION_MINOR, VERSION_PATCH

# Export version information to make it available at the package level
__all__ = ['__version__', 'VERSION_FULL', 'VERSION_SHORT', 'VERSION_MAJOR', 'VERSION_MINOR', 'VERSION_PATCH']